#  Copyright (c) 2021. Davi Pereira dos Santos
#  This file is part of the lange project.
#  Please respect the license - more about this in the section (*) below.
#
#  lange is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  lange is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with lange.  If not, see <http://www.gnu.org/licenses/>.
#
#  (*) Removing authorship by any means, e.g. by distribution of derived
#  works or verbatim, obfuscated, compiled or rewritten versions of any
#  part of this work is a crime and is unethical regarding the effort and
#  time spent here.
#  Relevant employers or funding agencies will be notified accordingly.

from lange.abs.progression import Progression


class AP(Progression):
    """Arithmetic Progression. A flexible inductive sequence of numbers based on Haskell interval notation.

    maxdigits: maximum amount of digits verified during precision detection
                (includes numbers to the left and to the right of the decimal separator, if any)

    Usage:
        >>> print(AP(5))
        [5]
        >>> print(AP(5, 1))
        [5 1]
        >>> print(AP(1, 3, ..., 10))
        [1 3 5 7 9]
        >>> ap = AP(0.6, 0.8, ..., 1.2)
        >>> ap
        [0.6 0.8 .+. 1.2]
        >>> print(ap)
        [0.6 0.8 1.0 1.2]
        >>> ap[1]
        0.8
        >>> print(AP(0.2, 4.1, 0.9))
        [0.2 4.1 0.9]
        >>> ap = AP(0.6, 0.8, ...)
        >>> ap
        [0.6 0.8 .+. inf]
        >>> print(ap[:5])
        [0.6 0.8 1.0 1.2 1.4]
        >>> AP()
        []
        >>> print(AP())
        []

    Usage (square brackets syntax):
        >>> from lange import ap
        >>> (ap[0.6, 0.8, ..., 2])
        [0.6 0.8 .+. 2.0]
        >>> ap[1, 2]
        [1 2 .+. 2]
        >>> list(ap[1, 2])
        [1, 2]
        >>> ap[1]
        [1 1 .+. 1]
        >>> list(ap[1])
        [1]
        >>> ap[3, 1, 6, 7, 0]
        [3 1 6 7 0]
        >>> list(ap[3, 1, 6, 7, 0])
        [3, 1, 6, 7, 0]
        >>> ap[4, 5, 2, 9, 4, 7, 3, 3][2:6]
        [2 9 4 7]
        >>> ap[4, 5, 2, 9, 4, 7, 3, 3][5]
        [7 7 .+. 7]
        >>> list(ap[3, 2, 6666, 7, 8, 6, 5555, 7, 3, 9][2:9:4])
        [6666, 5555]
        >>> print(ap[0.1, 0.2, ...][2:8])
        [0.3 0.4 0.5 0.6 0.7 0.8]
        >>> ap[0.1, 0.2, ...][2:8:2]
        [0.3 0.5 .+. 0.7]
    """

    def __init__(self, *args, maxdigits=28):
        super().__init__(
            "+",
            0,
            lambda a, b: a + b,
            lambda a, b: a - b,
            lambda a, b: a * b,
            lambda a, b, c: (b - a) / c,
            args,
            maxdigits,
        )

    def __reduce__(self):  # pragma: no cover
        return AP, self.args + (self.maxdigits,)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['lange', 'lange.abs']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'lange',
    'version': '0.220727.5',
    'description': 'Haskell-like intervals for Python',
    'long_description': '![test](https://github.com/davips/lange/workflows/test/badge.svg)\n[![codecov](https://codecov.io/gh/davips/lange/branch/main/graph/badge.svg)](https://codecov.io/gh/davips/lange)\n<a href="https://pypi.org/project/lange">\n<img src="https://img.shields.io/github/v/release/lange/lange?display_name=tag&sort=semver&color=blue" alt="github">\n</a>\n![Python version](https://img.shields.io/badge/python-3.8%20%7C%203.10-blue.svg)\n[![license: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)\n\n<!--- [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.5501845.svg)](https://doi.org/10.5281/zenodo.5501845) --->\n[![arXiv](https://img.shields.io/badge/arXiv-2109.06028-b31b1b.svg?style=flat-square)](https://arxiv.org/abs/2109.06028)\n[![API documentation](https://img.shields.io/badge/doc-API%20%28auto%29-a0a0a0.svg)](https://davips.github.io/lange)\n\n# lange\nLazy lists (i.e. Haskell-like ranges) for Python.\n\n## Installation\n### from package\n```bash\n# Set up a virtualenv. \npython3 -m venv venv\nsource venv/bin/activate\n\n# Install from PyPI\npip install lange\n```\n\n### from source\n```bash\ncd my-project\ngit clone https://github.com/davips/lange ../lange\npip install -e ../lange\n```\n\n\n### Features\n * Stable floating-point range generation, e.g.: `0.8 - 0.6 == 0.2` up to 28 digits (customizable).\n * Infinite `[1 2 ...]` or bounded.\n * O(1) access/evaluation `lst[3443]`\n\n\n### Examples\n\n**Arithmetic Progression**\n<details>\n<p>\n\n```python3\n\n# Bounded\nfrom lange import ap\nprint(ap[0.4, 0.8, ..., 2])\n"""\n[0.4 0.8 1.2 1.6 2.0]\n"""\n```\n\n```python3\n\n# Infinite + slicing\nprog = ap[0.4, 0.8, ...]\nprint(prog[:5])\n"""\n[0.4 0.8 1.2 1.6 2.0]\n"""\n```\n\n```python3\n\n# As list\nprint(list(prog[:5]))\n"""\n[0.4, 0.8, 1.2, 1.6, 2.0]\n"""\n```\n\n```python3\n\nprint(prog[:5].l)\n"""\n[0.4, 0.8, 1.2, 1.6, 2.0]\n"""\n```\n\n\n</p>\n</details>\n\n**Geometric Progression**\n<details>\n<p>\n\n```python3\n\n# Bounded\nfrom lange import gp\nprint(gp[0.4, 0.8, ..., 2])\n"""\n[0.4 0.8 1.6]\n"""\n```\n\n```python3\n\n# Infinite + slicing\nprog = gp[0.4, 0.8, ...]\nprint(prog[:5])\n"""\n[0.4 0.8 1.6 3.2 6.4]\n"""\n```\n\n```python3\n\n# As list\nprint(list(prog[:5]))\n"""\n[0.4, 0.8, 1.6, 3.2, 6.4]\n"""\n```\n\n```python3\n\nprint(prog[:5].l)\n"""\n[0.4, 0.8, 1.6, 3.2, 6.4]\n"""\n```\n\n\n</p>\n</details>\n',
    'author': 'davips',
    'author_email': 'dpsabc@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
